import os
import glob

import click

from . import configure, nginx, pg, uwsgi
from .nginx import parser
from jmb.tools import logger
from jmb.tools import NGINX_ENABLED_DIR, LETSENCRYPT_CONF_DIR

configured_repos = {}


@click.group()
@click.pass_context
def conf(ctx):
    """create or modify a configuration"""


def add_repo(repo, conf_file):
    """Add a repo to register of configured repostories

    :arg repo: The repo to add
    :arg conf_file: the file where the conf is
    """
    configured_repos['repo'] = conf_file


def register_configured_repositories(ip=False, conf_dir=NGINX_ENABLED_DIR):
    """register all repositories in /etc/nginx/sites-enabled

    :arg ip: (boolean, default: False) repo. If True, verify that the URL resolve here
    """
    for filename in glob.glob('{}/*'.format(conf_dir)):
        logger.debug("Nginx conf: %s" % filename)
        if not os.path.isfile(filename):
            continue
        cf = parser.ParseConf(filename)
        for srv in cf.servers:
            configured_repos[srv.root] = srv.filename


def get_nginx_conf(root):
    """Return the nginx conf file for a specific root (repo)

    :arg root: the root of the django project
    """
    if not configured_repos:
        register_configured_repositories()
    return configured_repos.get(root)


conf.add_command(configure.edit)
conf.add_command(configure.certbot)
conf.add_command(nginx.ng)
conf.add_command(pg.pg)
conf.add_command(uwsgi.uwsgi)
