import os
import re
import glob

import click
from . import create, parser
# from .parser import ParseConf
from jmb.tools import logger
from jmb.tools import NGINX_ENABLED_DIR


@click.group()
def ng():
    "create configuration for nginx"

ng.add_command(create.proxy)
ng.add_command(create.angjs)


def find_files(pattern='.*', rootdir='.', mindepth=0, maxdepth=2):
    """Find file by pattern up to a maximum depth (aka find -maxdepth 3 -name ...)
    """
    root_depth = rootdir.rstrip(os.path.sep).count(os.path.sep) - 1
    for dirpath, dirs, files in os.walk(rootdir):
        depth = dirpath.count(os.path.sep) - root_depth
        if mindepth <= depth <= maxdepth:
            for filename in files:
                if re.match(pattern, filename):
                    yield os.path.join(dirpath, filename)
        elif depth > maxdepth:
            del dirs[:]  # too deep, don't recurs


def get_all_repositories(root_dir=".", maxdepth=2, active=True,
                         nginx_enabled_dir=NGINX_ENABLED_DIR):
    """Return a (generator) of all repositories, i.e.: directory under ``root_dir``

    :arg root_dir: the directory where project are searched for
    :arg maxdepth:how deep to search for the projects (so as to skip .src...
    :arg active: (boolean). If True, only projects that resolv (as ip) here. Not implemented
    :arg nginx_enabled_dir: (test purpouse)
    """
    from jmb.tools import conf
    if not conf.configured_repos:
        conf.register_configured_repositories(conf_dir=nginx_enabled_dir)
    root_dir = os.path.abspath(root_dir)
    logger.debug("Root dir: %s" % root_dir)

    for path in find_files(rootdir=root_dir, maxdepth=maxdepth, pattern="setup.py$"):
        logger.debug("Found file: %s" % path)
        directory = os.path.dirname(path)
        if directory in conf.configured_repos:
            yield directory


def is_repo_active(repo, ip=False):
    """return filename if nginx has a reference to that repo in sites-enabled

    :arg repo: repo
    :arg ip: (boolean, default: False) repo. If True, verify that the URL resolve here
    """
    for filename in glob.glob(NGINX_ENABLED_DIR + '/*'):
        cf = parser.ParseConf(filename)
        for srv in cf.servers:
            if srv.root == repo:
                return filename
