# coding: utf-8

import click
from jmb.tools.utils.functions import execute, execute0
from jmb.tools.utils import read
from .. import client

SQL = """
SELECT d.datname as "Name",
 pg_catalog.pg_get_userbyid(d.datdba) as "Owner",
 CASE WHEN pg_catalog.has_database_privilege(d.datname, 'CONNECT')
     THEN pg_catalog.pg_database_size(d.datname)::varchar(50)
     ELSE 'No Access'
END as "Size"
 FROM pg_catalog.pg_database d
 JOIN pg_catalog.pg_tablespace t on d.dattablespace = t.oid
 ORDER BY 1;
 """


class PgClusterInfo(object):

    def __init__(self, version, name, port, status, server=None):
        self.version = version
        self.name = name
        self.port = port
        self.status = status
        self.server = server

    def get_databases(self, user=None):
        if user:
            cmd = ['sudo', '-U', user]
        else:
            cmd = []

        cmd += 'psql', '-p', self.port, '-c', SQL, 'postgres'
        ret_code, out, err = execute0(cmd)

    def register(self, verbose=False):
        """Register this cluster in te database
        """
        data = {
            'code': '{}-{}-{}'.format(self.server, self.version, self.name),
            'version': self.version,
            'name': self.name,
            'port': self.port,
            'status': self.status,
            'server': self.server
        }
        if verbose:
            click.secho("Registered cluster {}/{}".format(
                data['version'], data['name']), fg='magenta')

        stats_client = client.StatsClient()
        stats_client.clusters.add_or_update(data)

    def __repr__(self):
        return '<PgCluster {version}/{name} on {port}>'.format(**vars(self))


@click.command('cluster')
@click.option('-d', '--databases', is_flag=True, default=False, help="retrieve database list")
@click.option('-u', '--user', default=None, help="retrieve data as user via sudo")
@click.pass_context
def list_clusters(ctx, databases, user):
    "List PostgreSQL clusters on this server"

    data = read.ServerHost().data
    stats_client = client.StatsClient()
    stats_client.servers.add_or_update(data)
    clusters = list(get_clusters(server=data['name']))
    for cluster in clusters:
        cluster.register(ctx.parent.parent.parent.verbose)

    if databases:
        for cls in clusters:
            cls.get_databases(user=user)


def get_clusters(server):

    ret_code, out, err = execute0('pg_lsclusters -h')
    for line in out.splitlines():
        ver, name, port, status, owner, data, logs = line.split()
        yield PgClusterInfo(ver, name, port, status, server)
