# coding: utf-8

import re
import glob
import socket

import pyquery
import click
import requests
from requests.auth import HTTPBasicAuth
from jmb import tools
from jmb.tools.conf.nginx import parser
from jmb.tools.utils import read
from jmb.tools import logger


@click.command('resolve')
@click.option('--ssl', is_flag=True, help="Test https even if not present in this nginx conf", default=False)
@click.option('--user', '-u', help="The user for basic authentication")
@click.option('--ip', help="the ip to be used when checking if server_name points correctly")
@click.option('--password', '-p', help="The pass for basic authentication")
@click.option('--enabled', '-e', is_flag=True, help="look for names in nginx enabled conf",
              default=False)
@click.option('--ns', help="The nameserver to use", default='8.8.8.8')
@click.option('--response', '-r', is_flag=True, default=False, help="Test the response")
@click.option('--classes', '-c', is_flag=True, default=False,
              help="Test the classes of server_names: ip is 'A', name is 'CNAME'")
@click.option('--follow-redirects', '-f', is_flag=True, default=False, help="Follow all redirects to get the real page")
@click.argument('nginx_conf_files', nargs=-1)
def test_resolve(ssl, user, password, follow_redirects, nginx_conf_files, ip,
                 ns, classes, response, enabled):
    """Test conf files: server names correctness

    """

    if ip and re.search('[a-z]', ip):
        ip = socket.gethostbyname(ip)
    if not ip:
        ip = read.ServerHost().get_ip()
    print("follow_redirects", follow_redirects)
    ctx = click.get_current_context().parent.parent
    ## may be cominfrom invoke on jmb test domain:
    if not hasattr(ctx, 'verbose'):
        ctx = ctx.parent

    if nginx_conf_files and enabled:
        msg = u"Opt enabled implies all certificates from sites-enabled!!! no args permitted"
        raise click.BadParameter(msg)

    if enabled:
        conf_files = glob.glob(tools.NGINX_ENABLED_DIR + '/*')
    else:
        conf_files = nginx_conf_files

    for filename in conf_files:
        logger.debug("Conf: %s" % filename)
        if ctx.verbose:
            click.secho(filename, fg='magenta')
        try:
            pc = parser.ParseConf(filename)
            for srv in pc.servers:
                if classes:
                    srv.test_all_name_classes(ip=ip, nameserver=ns)

                elif response:
                    srv.test_all_name_responses(
                        ip=ip, nameserver=ns, verbose=ctx.verbose,
                        follow_redirects=follow_redirects, ssl=ssl)

                else:
                    srv.test_all_names(ip=ip, nameserver=ns)
        except Exception:
            click.secho("   !!!! Was not able to parse file: {}".format(filename), fg='red')
            raise


def test_conf(filename, verbose=False, ip=None, nameserver=None):
    ctx = click.get_current_context().parent.parent
    if ctx.verbose:
        click.secho(filename, fg='magenta')
    pc = parser.ParseConf(filename)
    for srv in pc.servers:
        srv.test_all_names(ip, nameserver=nameserver)


def test_classes(filename, verbose=False, ip=None, nameserver=None):
        srv.test_all_names(ip, nameserver=nameserver)
